#ifndef AGUIX_H
#define AGUIX_H

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/keysym.h>

#include "aguixdefs.h"
#include "lowlevelfunc.h"
#include "message.h"
#include "util.h"

class AWindow;
class AGUIXFont;

enum {MES_WAIT,MES_GET};
enum {NON_GADGET,CLOSE_GADGET,BUTTON_GADGET,SHADOW_GADGET,CHOOSE_GADGET,LISTVIEW,ONELVC,MORELVC,STRING_GADGET};

struct aguix_windowlist {
  AWindow *window;
  struct aguix_windowlist *next;
  struct aguix_windowlist *prev;
};

class AGUIX {
public:
  AGUIX();
  ~AGUIX();
  int initX(int argc,char **argv,char *classname);
  int checkX();
  void closeX();
  int getDepth();
  Colormap getColormap();
  Display *getDisplay();
  int getScreen();
  void panic(const char *);
  void setFG(int color);
  void setBG(int color);
  void setFG(GC gc,int color);
  void setBG(GC gc,int color);
  int getCharHeight();
  int getCharWidth();
  void FillRectangle(Drawable buffer,int x,int y,int w,int h);
  void DrawText(Drawable buffer,const char *text,int x,int y);
  void DrawLine(Drawable buffer,int x1,int y1,int x2,int y2);
  void FillRectangle(Drawable buffer,GC gc,int x,int y,int w,int h);
  void DrawText(Drawable buffer,AGUIXFont *,const char *text,int x,int y);
  void DrawLine(Drawable buffer,GC gc,int x1,int y1,int x2,int y2);
  void ClearWin(Window win);
  void SetWindowBG(Window win,int color);
  int getFontBaseline();
  int AddColor(int red,int green,int blue);
  void removeLastColor();
  int changeColor(int,int,int,int);
  int getMaxCols();
  unsigned long getPixel(int color);
  char *getClassname();
  Atom *getCloseAtom();
  void insertWindow(AWindow *win);
  void removeWindow(AWindow *win);
  AGMessage *GetMessage(AWindow *parent);
  AGMessage *WaitMessage(AWindow *parent);
  void ReplyMessage(AGMessage *);
  Message *wait4mess(int mode,bool);
  Message *wait4messReact(int mode,bool);
  void copyArea(Drawable source,Drawable dest,int s_x,int s_y,int width,int height,int d_x,int d_y);
  void Flush();
  int getargc();
  char **getargv();
  void putAGMsg(AGMessage *msg);
  AGMessage *getAGMsg();
  void WindowtoFront(Window);
  void WindowtoBack(Window);
  AGUIXFont *getFont(char*);
  int setFont(char*);
  void ExposeHandler(Message*);
  AWindow *findAWindow(Window);
  char *getNameOfKey(KeySym,unsigned int);
  Window getGroupWin();
  int queryPointer(Window win,int *x,int *y);
  int queryRootPointer(int *x,int *y);
  int queryPointer(Window win,int *x,int *y,unsigned int *buttons);
private:
  Display *dsp;
  int scr;
  Colormap cmap;
  GC gc;
  Atom WM_delete_window;
  char *classname;
  struct aguix_windowlist *firstwin;
  unsigned long white,black;
  XFontStruct *myfont;
  AGUIXFont *mainfont;
  int initOK;
  int argc;
  char **argv;
  int CharHeight;
  int CharWidth;
  unsigned long ColBuf[256];
  int colors;
  XEvent LastEvent;
  List *wins;
  List *messages;
  List *fonts;
  Window groupwin;
  void createGroupWin();
  void destroyGroupWin();
  
  bool ReactMessage(Message*,AWindow*);
  void buildAGMessage();
  
  bool privatecmap;
  void changeColormap();
};

class AGUIXFont
{
public:
  AGUIXFont(AGUIX*);
  ~AGUIXFont();
  int setFont(char*);
  void removeFont();
  GC getGC();
  char *getName();
  int getCharHeight();
  int getCharWidth();
  int getBaseline();
  XFontStruct *getFontStruct();
private:
  char *name;
  GC gc;
  AGUIX *aguix;
  XFontStruct *font;
  int charWidth,charHeight;
};

#endif

