#include "button.h"
#include "awindow.h"
#include "guielement.h"

const char *Button::type="Button";

Button::~Button()
{
  if(text[0]!=NULL) _freesafe(text[0]);
  if(text[1]!=NULL) _freesafe(text[1]);
  if(parent!=NULL) {
    parent->removeSubWin(win);
  }
}

Button::Button(AGUIX *aguix):GUIElement(aguix)
{
  this->dual=false;
  this->data=0;
  this->state=0;
  this->instate=0;
  this->text[0]=(char*)_allocsafe(1);
  this->text[1]=(char*)_allocsafe(1);
  this->text[0][0]=0;
  this->text[1][0]=0;
  this->fg[0]=0;
  this->fg[1]=0;
  this->bg[0]=0;
  this->bg[1]=0;
  lastbg=-1;
  showdual=true;
  font=NULL;
}

Button::Button(AGUIX *aguix,int x,int y,int width,
               const char *text,int fg,int bg,int data):GUIElement(aguix)
{
  this->x=x;
  this->y=y;
  this->w=width;
  this->h=aguix->getCharHeight()+4;
  this->dual=false;
  this->data=data;
  this->state=0;
  this->instate=0;
  this->text[0]=dupstring(text);
  this->text[1]=(char*)_allocsafe(1);
  this->text[1][0]=0;
  this->fg[0]=fg;
  this->fg[1]=0;
  this->bg[0]=bg;
  this->bg[1]=0;
  lastbg=-1;
  showdual=true;
  font=NULL;
}

Button::Button(AGUIX *aguix,int x,int y,int width,int height,
               const char *text,int fg,int bg,int data):GUIElement(aguix)
{
  this->x=x;
  this->y=y;
  this->w=width;
  this->h=height;
  this->dual=false;
  this->data=data;
  this->state=0;
  this->instate=0;
  this->text[0]=dupstring(text);
  this->text[1]=(char*)_allocsafe(1);
  this->text[1][0]=0;
  this->fg[0]=fg;
  this->fg[1]=0;
  this->bg[0]=bg;
  this->bg[1]=0;
  lastbg=-1;
  showdual=true;
  font=NULL;
}

Button::Button(AGUIX *aguix,int x,int y,int width,
               const char *text_normal,const char *text_high,
               int fg_normal,int fg_high,
               int bg_normal,int bg_high,int data):GUIElement(aguix)
{
  this->x=x;
  this->y=y;
  this->w=width;
  this->h=aguix->getCharHeight()+4;
  this->dual=true;
  this->data=data;
  this->state=0;
  this->instate=0;
  this->text[0]=dupstring(text_normal);
  this->text[1]=dupstring(text_high);
  this->fg[0]=fg_normal;
  this->fg[1]=fg_high;
  this->bg[0]=bg_normal;
  this->bg[1]=bg_high;
  lastbg=-1;
  showdual=true;
  font=NULL;
}

char *Button::getText(int index)
{
  if((index<0)||(index>1)) return NULL;
  return dupstring(text[index]);
}

void Button::setText(int index,const char *new_text)
{
  if((index<0)||(index>1)) return;
  if(text[index]!=NULL) _freesafe(text[index]);
  text[index]=dupstring(new_text);
  redraw();
}

bool Button::getDualState()
{
  return dual;
}

void Button::setDualState(bool new_dual)
{
  dual=new_dual;
  redraw();
}

void Button::setFG(int index,int fg)
{
  if((index<0)||(index>1)) return;
  if(fg<(aguix->getMaxCols())) {
    this->fg[index]=fg;
  }
  redraw();
}

int Button::getFG(int index)
{
  if((index<0)||(index>1)) return 0;
  return fg[index];
}

void Button::setBG(int index,int bg)
{
  if((index<0)||(index>1)) return;
  if(bg<(aguix->getMaxCols())) {
    this->bg[index]=bg;
  }
  redraw();
}

int Button::getBG(int index)
{
  if((index<0)||(index>1)) return 0;
  return bg[index];
}

int Button::getData()
{
  return data;
}

void Button::setData(int data)
{
  this->data=data;
}

void Button::redraw()
{
  int newbg;
  GC usegc;
  if(parent==NULL) return;
  if(win==0) return;
  if((dual==false)||(state<2)) {
    newbg=bg[0];
  } else {
    newbg=bg[1];
  }
//  if(newbg!=lastbg) {
    aguix->SetWindowBG(win,newbg);
    lastbg=newbg;
//  }
  aguix->ClearWin(win);
  if(font==NULL) usegc=0; else usegc=font->getGC();
  if(state==0) {
    aguix->setFG(usegc,2);
  } else {
    aguix->setFG(usegc,1);
  }
  aguix->DrawLine(win,usegc,0,h-1,0,0);
  aguix->DrawLine(win,usegc,0,0,w-1,0);
  if(state==0) {
    aguix->setFG(usegc,1);
  } else {
    aguix->setFG(usegc,2);
  }
  aguix->DrawLine(win,usegc,0,h-1,w-1,h-1);
  aguix->DrawLine(win,usegc,w-1,h-1,w-1,1);
  if((dual==true)&&(showdual==true)) {
    aguix->setFG(usegc,2);
    int dw=w/6;
    int dw2=h/3;
    if(dw2<dw) dw=dw2;
    aguix->DrawLine(win,usegc,w-dw+1,1,w-1,dw-1);
  }
  if((dual==false)||(state<2)) {
    aguix->setFG(usegc,fg[0]);
  } else {
    aguix->setFG(usegc,fg[1]);
  }
  int ch,cw;
  if(font==NULL) {
    ch=aguix->getCharHeight();
    cw=aguix->getCharWidth();
  } else {
    ch=font->getCharHeight();
    cw=font->getCharWidth();
  }
  int dy=h/2-ch/2;
  unsigned int maxdisplay=(w-2)/cw;
  char *tstr;
  if((dual==false)||(state<2)) {
    tstr=dupstring(text[0]);
  } else {
    tstr=dupstring(text[1]);
  }
  if(strlen(tstr)>maxdisplay) tstr[maxdisplay]=0;
  int dw=strlen(tstr)*cw;
  int dx=w/2-dw/2;
  if(font==NULL) {
    aguix->DrawText(win,tstr,dx,dy);
  } else {
    aguix->DrawText(win,font,tstr,dx,dy);
  }
  _freesafe(tstr);
  aguix->Flush();
}

void Button::flush()
{
}

void Button::setState(int state)
{
  if((dual==false)&&(state==2)) {
    this->state=1;
  } else {
    this->state=state;
  }
  redraw();
}

int Button::getState()
{
  return state;
}

bool Button::isInside(int px,int py)
{
  if((px>0)&&(px<=w)) {
    if((py>0)&&(py<=h)) return true;
  }
  return false;
}

bool Button::handleMessage(XEvent *E,Message *msg)
{
  bool returnvalue;
  returnvalue=false;
  if(msg->window!=win) return false;
  if((msg->type==ButtonPress)||(msg->type==ButtonRelease)) {
    if((msg->button==Button3)||(msg->button==Button1)) {
      int mx,my;
      mx=msg->mousex;
      my=msg->mousey;
      if(msg->type==ButtonPress) {
        if(msg->window==win) {
          AGMessage *agmsg=(AGMessage*)_allocsafe(sizeof(AGMessage));
          agmsg->type=AG_BUTTONPRESSED;
          agmsg->button.button=this;
          if((msg->button==Button3)&&(dual==true)) {
            setState(2);
            instate=2;
            agmsg->button.state=2;
          } else {
            setState(1);
            instate=1;
            agmsg->button.state=1;
          }
          aguix->putAGMsg(agmsg);
          returnvalue=true;
        }
      } else {
        if((state!=0)&&(instate!=0)) {
          AGMessage *agmsg=(AGMessage*)_allocsafe(sizeof(AGMessage));
          agmsg->type=AG_BUTTONRELEASED;
          agmsg->button.button=this;
          agmsg->button.state=instate;
          aguix->putAGMsg(agmsg);
          agmsg=(AGMessage*)_allocsafe(sizeof(AGMessage));
          agmsg->type=AG_BUTTONCLICKED;
          agmsg->button.button=this;
          agmsg->button.state=instate;
          aguix->putAGMsg(agmsg);
        }
        if(instate!=0) {
          setState(0);
          instate=0;
          returnvalue=true;
        }
      }
    }
  } else if(msg->type==EnterNotify) {
    // alles hier und alles mit instate wird benutzt, damit Button sich anpat, wenn
    // Mauszeiger im Button oder auerhalb des Buttons ist
    if(instate!=0) {
      if(state!=instate) {
        setState(instate);
      }
    }
  } else if(msg->type==LeaveNotify) {
    // alles hier und alles mit instate wird benutzt, damit Button sich anpat, wenn
    // Mauszeiger im Button oder auerhalb des Buttons ist
    if(instate!=0) {
      setState(0);
    }
  } else if(msg->type==Expose) {
    redraw();
  }
  if(returnvalue==true) {
    // jetzt noch die Message mit den Werten fllen
    msg->gadget=this;
    msg->gadgettype=BUTTON_GADGET;
  }
//  return returnvalue;
  return false; // see cyclebutton.cc
}

bool Button::getShowDual()
{
  return showdual;
}

void Button::setShowDual(bool new_val)
{
  if(showdual!=new_val) {
    showdual=new_val;
    redraw();
  } else showdual=new_val;
}

int Button::setFont(char *fontname)
{
  font=aguix->getFont(fontname);
  if(font==NULL) return -1;
  return 0;
}

const char *Button::getType()
{
  return type;
}

bool Button::isType(const char *qtype)
{
  if(strcmp(type,qtype)==0) return true;
  return false;
}
