#include "lowlevelfunc.h"

#ifdef USE_MEM_SYSTEM
typedef struct _safemem {void *ptr;
                         size_t size;
                         struct _safemem *next;} SafeMem;
SafeMem *firstmemstr;
unsigned long maxmemusage,memusage;
#endif

void *_allocsafe(size_t size)
{ /* gibt Speicher zurck oder beendet sofort das Programm */
  void *ptr;
#ifdef USE_MEM_SYSTEM
  SafeMem *tsmptr,*tsmptr2;
#endif

  ptr=(void*)malloc(size);
  if(ptr==NULL) {
    fprintf(stderr, "No Memory!!!!Aborting\n");
    exit(1);
  }
#ifdef USE_MEM_SYSTEM
  tsmptr=(SafeMem*)malloc(sizeof(SafeMem));
  tsmptr->ptr=ptr;
  tsmptr->size=size;
  tsmptr->next=NULL;
  if(firstmemstr==NULL) {
    firstmemstr=tsmptr;
  } else {
    tsmptr2=firstmemstr;
    while(tsmptr2->next!=NULL) tsmptr2=tsmptr2->next;
    tsmptr2->next=tsmptr;
  }
  memusage+=size;
  if(memusage>maxmemusage) maxmemusage=memusage;
#endif
  return ptr;
}

void waittime(unsigned long msec)
{
  struct timespec wt;
  wt.tv_nsec=msec*1000000;
  wt.tv_sec=0;
  nanosleep(&wt,NULL);
}

#ifdef USE_MEM_SYSTEM
void _freesafe(void *p)
{
  SafeMem *tsmptr,*found,*prefound;

  tsmptr=firstmemstr;
  found=NULL;
  prefound=NULL;
  while(tsmptr!=NULL) {
    if(tsmptr->ptr==p) {
      found=tsmptr;
      break;
    }
    prefound=tsmptr;
    tsmptr=tsmptr->next;
  }
  if(found!=NULL) {
    memusage-=found->size;
    if(firstmemstr==found) {
      firstmemstr=found->next;
      free(found);
    } else {
      prefound->next=found->next;
      free(found);
    }
  } else {
    printf("Freigabe eines unauthorisierten Speichers!\n");
  }
  free(p);
}

void _memsystemcheck()
{
  SafeMem *tsmptr;
  int nr;

  tsmptr=firstmemstr;
  nr=0;
  while(tsmptr!=NULL) {
    nr++;
/*    printf("lasse nr %d free:\n",nr);
    free(tsmptr->ptr);*/
    tsmptr=tsmptr->next;
  }
#ifdef DEBUG
  printf("Es gibt noch %d nicht freigelassene Speicherstcke!\n",nr);
  printf("Maximal benutzter Speicher:%ld Bytes\n",maxmemusage);
  printf("Noch benutzter Speicher:%ld Bytes\n",memusage);
#else
  if(nr>0) {
    printf("Es gibt noch %d nicht freigelassene Speicherstcke!\n",nr);
  }
#endif
}

void _memsysteminit()
{
  printf("Init Mem-System\n");
  firstmemstr=NULL;
  maxmemusage=0;
  memusage=0;
}
#endif

char *dupstring(const char *str)
{
  char *tstr;
  tstr=(char*)_allocsafe(strlen(str)+1);
  strcpy(tstr,str);
  return tstr;
}

void MakeLong2NiceStr(char *buffer,unsigned long size)
{
  unsigned long msize,ksize,rsize;
  int offset;
  msize=size/1000000;
  ksize=size%1000000;
  ksize/=1000;
  rsize=size%1000;
  buffer[0]=0;
  if(msize>0) {
    sprintf(buffer,"%ld.",msize);
  }
  offset=strlen(buffer);
  if(offset>0) {
    sprintf(buffer+offset,"%03ld.",ksize);
  } else {
    if(ksize>0) {
      sprintf(buffer,"%ld.",ksize);
    }
  }
  offset=strlen(buffer);
  if(offset>0) {
    sprintf(buffer+offset,"%03ld",rsize);
  } else {
    sprintf(buffer,"%ld",rsize);
  }
}

int LongSize(unsigned long value)
{
  int size;
  unsigned long x;
  x=value;
  size=1;
  if(x>999) size++;
  if(x>999999) size++;
  while(x>9) {
    size++;
    x=x/10;
  }
  return size;
}

char *catstring(const char *str1,const char *str2)
{
  char *tstr;
  tstr=(char*)_allocsafe(strlen(str1)+strlen(str2)+1);
  strcpy(tstr,str1);
  strcpy(tstr+strlen(str1),str2);
  return tstr;
}
