#include "solidbutton.h"
#include "lowlevelfunc.h"
#include "awindow.h"

const char *SolidButton::type="SolidButton";

SolidButton::~SolidButton()
{
  if(text!=NULL) _freesafe(text);
}

SolidButton::SolidButton(AGUIX *aguix):GUIElement(aguix)
{
  this->state=false;
  this->text=(char*)_allocsafe(1);
  this->text[0]=0;
  this->fg=0;
  this->bg=0;
  lastbg=-1;
  font=NULL;
}

SolidButton::SolidButton(AGUIX *aguix,int x,int y,int width,
                         const char *text,int fg,int bg,bool state):GUIElement(aguix)
{
  this->x=x;
  this->y=y;
  this->w=width;
  this->h=aguix->getCharHeight()+4;
  this->state=state;
  this->text=dupstring(text);
  this->fg=fg;
  this->bg=bg;
  lastbg=-1;
  font=NULL;
}

SolidButton::SolidButton(AGUIX *aguix,int x,int y,int width,int height,
                         const char *text,int fg,int bg,bool state):GUIElement(aguix)
{
  this->x=x;
  this->y=y;
  this->w=width;
  this->h=height;
  this->state=state;
  this->text=dupstring(text);
  this->fg=fg;
  this->bg=bg;
  lastbg=-1;
  font=NULL;
}

char *SolidButton::getText()
{
  if(text!=NULL) return dupstring(text);
  return NULL;
}

void SolidButton::setText(const char *new_text)
{
  if(text!=NULL) _freesafe(text);
  text=dupstring(new_text);
  redraw();
}

void SolidButton::setFG(int fg)
{
  if(fg<(aguix->getMaxCols())) {
    this->fg=fg;
    redraw();
  }
}

int SolidButton::getFG()
{
  return fg;
}

void SolidButton::setBG(int bg)
{
  if(bg<(aguix->getMaxCols())) {
    this->bg=bg;
    redraw();
  }
}

int SolidButton::getBG()
{
  return bg;
}

bool SolidButton::getState()
{
  return state;
}

void SolidButton::setState(bool state)
{
  this->state=state;
  redraw();
}

void SolidButton::redraw()
{
  GC usegc;
  if(parent==NULL) return;
  if(win==0) return;
//  if(lastbg!=bg) {
    aguix->SetWindowBG(win,bg);
    lastbg=bg;
//  }
  aguix->ClearWin(win);
  if(font==NULL) usegc=0; else usegc=font->getGC();
  if(state==true) {
    aguix->setFG(usegc,1);
  } else {
    aguix->setFG(usegc,2);
  }
  aguix->DrawLine(win,usegc,0,h-1,0,0);
  aguix->DrawLine(win,usegc,0,0,w-1,0);
  if(state==true) {
    aguix->setFG(usegc,2);
  } else {
    aguix->setFG(usegc,1);
  }
  aguix->DrawLine(win,usegc,0,h-1,w-1,h-1);
  aguix->DrawLine(win,usegc,w-1,h-1,w-1,1);
  aguix->setFG(usegc,fg);
  int ch,cw;
  if(font==NULL) {
    ch=aguix->getCharHeight();
    cw=aguix->getCharWidth();
  } else {
    ch=font->getCharHeight();
    cw=font->getCharWidth();
  }
  int dy=h/2-ch/2;
  unsigned int maxdisplay=(w-2)/cw;
  char *tstr;
  tstr=dupstring(text);
  if(strlen(tstr)>maxdisplay) tstr[maxdisplay]=0;
  int dw=strlen(tstr)*(aguix->getCharWidth());
  int dx=w/2-dw/2;
  if(font==NULL) aguix->DrawText(win,tstr,dx,dy);
  else aguix->DrawText(win,font,tstr,dx,dy);
  _freesafe(tstr);
  aguix->Flush();
}

void SolidButton::flush()
{
}

bool SolidButton::handleMessage(XEvent *E,Message *msg)
{
  if((msg->type==Expose)&&(msg->window==win)) {
    redraw();
  }
  return false;
}

int SolidButton::setFont(char *fontname)
{
  font=aguix->getFont(fontname);
  if(font==NULL) return -1;
  return 0;
}

const char *SolidButton::getType()
{
  return type;
}

bool SolidButton::isType(const char *qtype)
{
  if(strcmp(type,qtype)==0) return true;
  return false;
}
