#include "stringgadget.h"
#include "awindow.h"
#include "guielement.h"

const char *StringGadget::type="StringGadget";

StringGadget::~StringGadget()
{
  _freesafe(text);
  if(parent!=NULL) {
    parent->removeSubWin(twin);
    parent->removeSubWin(win);
  }
}

StringGadget::StringGadget(AGUIX *aguix):GUIElement(aguix)
{
  data=0;
  w=100;
  h=100;
  text=(char*)_allocsafe(1);
  text[0]=0;
  cursorpos=0;
  xoffset=0;
  active=false;
  tbg=-1;
  bgset=false;
  selstart=selend=cursorpos;
  font=NULL;
  forbidPosChange=false;
  strongkeycapture=true;
}

StringGadget::StringGadget(AGUIX *aguix,int x,int y,int width,int height,const char *text,int data):GUIElement(aguix)
{
  this->data=data;
  this->x=x;
  this->y=y;
  this->w=width;
  this->h=height;
  xoffset=0;
  cursorpos=0;
  active=false;
  this->text=dupstring(text);
  tbg=-1;
  bgset=false;
  selstart=selend=cursorpos;
  font=NULL;
  forbidPosChange=false;
  strongkeycapture=true;
}

StringGadget::StringGadget(AGUIX *aguix,int x,int y,int width,const char *text,int data):GUIElement(aguix)
{
  this->data=data;
  this->x=x;
  this->y=y;
  this->w=width;
  this->h=aguix->getCharHeight()+2+6;
  xoffset=0;
  cursorpos=0;
  xoffset=0;
  active=false;
  this->text=dupstring(text);
  tbg=-1;
  bgset=false;
  selstart=selend=cursorpos;
  font=NULL;
  forbidPosChange=false;
  strongkeycapture=true;
}

void StringGadget::resize(int w,int h)
{
  this->w=w;
  this->h=h;
  if(parent!=NULL) {
    parent->resizeSubWin(win,w,h);
  }
  updateWin();
  redraw();
}

int StringGadget::getData()
{
  return data;
}

void StringGadget::setData(int data)
{
  this->data=data;
}

void StringGadget::redraw()
{
  bredraw();
  tredraw();
}

void StringGadget::bredraw()
{
  aguix->SetWindowBG(win,parent->getBG());
  aguix->ClearWin(win);
  aguix->setFG(2);
  aguix->DrawLine(win,0,h-1,0,0);
  aguix->DrawLine(win,0,0,w-1,0);
  aguix->DrawLine(win,2,h-1-2,w-1-2,h-1-2);
  aguix->DrawLine(win,w-1-2,h-1-2,w-1-2,2);
  aguix->setFG(1);
  aguix->DrawLine(win,0,h-1,w-1,h-1);
  aguix->DrawLine(win,w-1,h-1,w-1,0);
  aguix->DrawLine(win,2,h-1-2,2,2);
  aguix->DrawLine(win,2,2,w-1-2,2);
}

void StringGadget::tredraw()
{
  int maxcol=aguix->getMaxCols();
  int newbg;
  GC usegc;
  if(active==true) {
    if(maxcol>7) {
      newbg=7;
    } else {
      newbg=0;
    }
  } else {
    newbg=0;
  }
  aguix->SetWindowBG(twin,newbg);
  tbg=newbg;
  aguix->ClearWin(twin);
  if(font==NULL) usegc=0; else usegc=font->getGC();
  int cw,ch;
  if(font==NULL) {
    ch=aguix->getCharHeight();
    cw=aguix->getCharWidth();
  } else {
    ch=font->getCharHeight();
    cw=font->getCharWidth();
  }
  char *tstr=(char*)_allocsafe(maxdisplay+1);
  int dx;
  dx=1;
  double f1=((h-6-2)/2)-(ch/2);
  int dy=(int)f1;
  int x1,x2;
  x1=(selstart<=selend?selstart:selend);
  x2=(selstart>selend?selstart:selend);
  if((x2==(int)strlen(text))&&(selstart!=selend)) x2--;
//  if((x1<=x2)&&(selstart>=0)) {
    // Eine Selektion
    if((xoffset<x1)&&((x1-xoffset)<=maxdisplay)) {
      // links vom x1 ist noch Text
      aguix->setFG(usegc,1);
      strncpy(tstr,text+xoffset,x1-xoffset);
      tstr[x1-xoffset]=0;
      if(font==NULL) aguix->DrawText(twin,tstr,dx,dy);
      else aguix->DrawText(twin,font,tstr,dx,dy);
      dx+=strlen(tstr)*cw;
    }
    int maxx=(x1<x2)?x2:x1;
    int minx=(x1<x2)?x1:x2;
    if(!((maxx<xoffset)||((minx-xoffset)>/*=*/maxdisplay))) {
      // die Selektion ist sichtbar
      minx=(x1<xoffset)?xoffset:x1;
      maxx=(x2<(xoffset+maxdisplay))?x2:(xoffset+maxdisplay-1);
      int tl=maxx-minx+1;
      strncpy(tstr,text+minx,tl);
      tstr[tl]=0;
      if(active==true) {
        aguix->setFG(usegc,1);
      } else {
        aguix->setFG(usegc,0);
      }
      aguix->FillRectangle(twin,usegc,dx,dy,tl*cw,ch);
      if(active==true) {
        if(maxcol>7) {
          aguix->setFG(usegc,7);
        } else {
          aguix->setFG(usegc,0);
        }
      } else {
        aguix->setFG(usegc,1);
      }
      if(font==NULL) aguix->DrawText(twin,tstr,dx,dy);
      else aguix->DrawText(twin,font,tstr,dx,dy);
      dx+=tl*cw;
    }
    if((x2-xoffset)<(maxdisplay-1)) {
      int t1;
      t1=x2+1;
      if(t1<x2) t1=x2;
      if((int)strlen(text)>=t1) {
        strncpy(tstr,text+t1,maxdisplay-(x2-xoffset)-1);
        tstr[maxdisplay-(x2-xoffset)-1]=0;
        aguix->setFG(usegc,1);
	if(font==NULL) aguix->DrawText(twin,tstr,dx,dy);
	else aguix->DrawText(twin,font,tstr,dx,dy);
      }
    }
/*  } else {
    if((xoffset<cursorpos)&&((cursorpos-xoffset)<=maxdisplay)) {
      / links vom Cursor ist noch Text
      aguix->setFG(1);
      strncpy(tstr,text+xoffset,cursorpos-xoffset);
      tstr[cursorpos-xoffset]=0;
      aguix->DrawText(twin,tstr,dx,dy);
      dx+=strlen(tstr)*aguix->getCharWidth();
    }
    if((xoffset<=cursorpos)&&((cursorpos-xoffset)<maxdisplay)) {
      // der Cursor ist sichtbar
      tstr[1]=0;
      tstr[0]=text[cursorpos];
      if(active==true) {
        aguix->setFG(1);
      } else {
        aguix->setFG(0);
      }
      aguix->FillRectangle(twin,dx,dy,aguix->getCharWidth(),aguix->getCharHeight());
      if(active==true) {
        if(maxcol>7) {
          aguix->setFG(7);
        } else {
          aguix->setFG(0);
        }
      } else {
        aguix->setFG(1);
      }
      aguix->DrawText(twin,tstr,dx,dy);
      dx+=aguix->getCharWidth();
    }
    if((cursorpos-xoffset)<(maxdisplay-1)) {
      int t1;
      t1=cursorpos+1;
      if(t1<xoffset) t1=xoffset;
      if((int)strlen(text)>=t1) {
        strncpy(tstr,text+t1,maxdisplay-(cursorpos-xoffset)-1);
        tstr[maxdisplay-(cursorpos-xoffset)-1]=0;
        aguix->setFG(1);
        aguix->DrawText(twin,tstr,dx,dy);
      }
    }
  }*/
  _freesafe(tstr);
  aguix->Flush();
}

void StringGadget::flush()
{
}

bool StringGadget::isInside(int px,int py)
{
/*  if((px>x)&&(px<=(x+width))) {
    if((py>y)&&(py<=(y+height))) return true;
  }*/
  return false;
}

bool StringGadget::handleMessage(XEvent *E,Message *msg)
{
  bool returnvalue;
  returnvalue=false;
  int cw;
  int tx,ty;
  
  if(font==NULL) {
    cw=aguix->getCharWidth();
  } else {
    cw=font->getCharWidth();
  }
  if(active==true) {
    if(msg->type==ButtonPress) {
      if(msg->window!=twin) {
        active=false;
        tredraw();
//        returnvalue=true;
        AGMessage *agmsg=(AGMessage*)_allocsafe(sizeof(AGMessage));
        agmsg->type=AG_STRINGGADGET_DEACTIVATE;
        agmsg->stringgadget.sg=this;
        aguix->putAGMsg(agmsg);
      } else {
        if(forbidPosChange==false) {
	  aguix->queryPointer(msg->window,&tx,&ty);
          int nc=tx/cw;
          cursorpos=nc+xoffset;
          if(cursorpos>=(int)strlen(text)) cursorpos=(int)strlen(text);
          if(cursorpos<0) cursorpos=0;
          selstart=selend=cursorpos;
          if(msg->button==Button2) insertSelection();
          redraw();
	}
      }
    } else {
      if(msg->type==KeyPress) {
        ignoreRelease=false;
        int keystate=KEYSTATEMASK(msg->keystate);
        if((msg->key==XK_Right)||
	   (msg->key==XK_End)||
	   ((msg->key==XK_e)&&(keystate==ControlMask))||
	   ((msg->key==XK_f)&&(keystate==ControlMask))) {
	  if(strongkeycapture==true) returnvalue=true;
	  if(forbidPosChange==false) {
            if((((keystate==0)||(keystate==ShiftMask))&&
	       (msg->key==XK_Right))||
	       ((msg->key==XK_f)&&(keystate==ControlMask))) {
              if(cursorpos<(int)strlen(text)) {
                cursorpos++;
                if((cursorpos-xoffset)>=maxdisplay) xoffset++;
	        if(keystate!=ShiftMask) selstart=selend=cursorpos;
  	        else {
 	          selend=cursorpos;
  	          applySelection();
                }
                tredraw();
                returnvalue=true;
              }
            } else if(((keystate&Mod1Mask)==Mod1Mask)||
	              (msg->key==XK_End)||
		      ((msg->key==XK_e)&&(keystate==ControlMask))) {
              cursorpos=(int)strlen(text);
              while((cursorpos-xoffset)>=maxdisplay) xoffset++;
  	      if((keystate&ShiftMask)==0) selstart=selend=cursorpos;
  	      else selend=cursorpos;
              tredraw();
              returnvalue=true;
            }
            AGMessage *agmsg=(AGMessage*)_allocsafe(sizeof(AGMessage));
            agmsg->type=AG_STRINGGADGET_CURSORCHANGE;
            agmsg->stringgadget.sg=this;
            aguix->putAGMsg(agmsg);
	  }
        } else if((msg->key==XK_Left)||
	          (msg->key==XK_Home)||
		  ((msg->key==XK_a)&&(keystate==ControlMask))||
		  ((msg->key==XK_b)&&(keystate==ControlMask))) {
	  if(strongkeycapture==true) returnvalue=true;
          if(forbidPosChange==false) {
            if((((keystate==0)||(keystate==ShiftMask))&&
	       (msg->key==XK_Left))||
	       ((msg->key==XK_b)&&(keystate==ControlMask))) {
              if(cursorpos>0) {
                cursorpos--;
                if((cursorpos-xoffset)<0) xoffset--;
	        if(keystate!=ShiftMask) selstart=selend=cursorpos;
	        else {
	          selend=cursorpos;
	  	  applySelection();
  	        }
                tredraw();
                returnvalue=true;
              }
            } else if(((keystate&Mod1Mask)==Mod1Mask)||
	              (msg->key==XK_Home)||
		      ((msg->key==XK_a)&&(keystate==ControlMask))) {
              xoffset=0;
              cursorpos=0;
	      if((keystate&ShiftMask)==0) selstart=selend=cursorpos;
  	      else selend=cursorpos;
              tredraw();
              returnvalue=true;
            }
            AGMessage *agmsg=(AGMessage*)_allocsafe(sizeof(AGMessage));
            agmsg->type=AG_STRINGGADGET_CURSORCHANGE;
            agmsg->stringgadget.sg=this;
            aguix->putAGMsg(agmsg);
	  }
        } else if((msg->key==XK_Delete)||((msg->key==XK_d)&&(keystate==ControlMask))) {
          int len=strlen(text);
	  if(strongkeycapture==true) returnvalue=true;
	  if(selstart!=selend) {
	    removeSelection();
	    tredraw();
	  } else if((cursorpos<=len)&&(len>0)) {
            char *tstr=(char*)_allocsafe(len);
            strncpy(tstr,text,cursorpos);
            tstr[cursorpos]=0;
            if(cursorpos<len) strcat(tstr,text+cursorpos+1);
            _freesafe(text);
            text=tstr;
            tredraw();
            returnvalue=true;
            AGMessage *agmsg=(AGMessage*)_allocsafe(sizeof(AGMessage));
            agmsg->type=AG_STRINGGADGET_CONTENTCHANGE;
            agmsg->stringgadget.sg=this;
            aguix->putAGMsg(agmsg);
          }
        } else if((msg->key==XK_BackSpace)||((msg->key==XK_h)&&(keystate==ControlMask))) {
          int len=strlen(text);
	  if(strongkeycapture==true) returnvalue=true;
	  if(selend!=selstart) {
	    removeSelection();
	    tredraw();
	  } else if(cursorpos>0) {
            char *tstr=(char*)_allocsafe(len);
            tstr[0]=0;
            if(cursorpos>1) {
              strncpy(tstr,text,cursorpos-1);
              tstr[cursorpos-1]=0;
            }
            strcat(tstr,text+cursorpos);
            _freesafe(text);
            text=tstr;
            cursorpos--;
            if((cursorpos-xoffset)<0) xoffset--;
	    selstart=selend=cursorpos;
            tredraw();
            returnvalue=true;
            AGMessage *agmsg=(AGMessage*)_allocsafe(sizeof(AGMessage));
            agmsg->type=AG_STRINGGADGET_CONTENTCHANGE;
            agmsg->stringgadget.sg=this;
            aguix->putAGMsg(agmsg);
          }
        } else if((msg->key>=XK_space)&&(msg->key<=0xff)) {
          int len=strlen(text);
	  if(selstart!=selend) {
	    removeSelection();
	    tredraw();
	  }
          char *tstr=(char*)_allocsafe(len+2);
          tstr[0]=0;
          strncpy(tstr,text,cursorpos);
          tstr[cursorpos]=0;
          len=strlen(tstr);
          tstr[len]=(char)msg->key;
          tstr[len+1]=0;
          strcat(tstr,text+cursorpos);
          _freesafe(text);
          text=tstr;
          cursorpos++;
          if((cursorpos-xoffset)>=maxdisplay) xoffset++;
	  selstart=selend=cursorpos;
          tredraw();
          returnvalue=true;
          AGMessage *agmsg=(AGMessage*)_allocsafe(sizeof(AGMessage));
          agmsg->type=AG_STRINGGADGET_CONTENTCHANGE;
          agmsg->stringgadget.sg=this;
          aguix->putAGMsg(agmsg);
        } else if(msg->key==XK_Return) {
	  if(strongkeycapture==true) returnvalue=true;
	}
      } else if((msg->type==KeyRelease)&&(ignoreRelease==false)) {
        if((msg->key==XK_Return)||(msg->key==XK_Escape)) {
          active=false;
	  selstart=selend=cursorpos;
          tredraw();
          returnvalue=true;
          AGMessage *agmsg=(AGMessage*)_allocsafe(sizeof(AGMessage));
          agmsg->type=AG_STRINGGADGET_DEACTIVATE;
          agmsg->stringgadget.sg=this;
          aguix->putAGMsg(agmsg);
          if(msg->key==XK_Return) {
            AGMessage *agmsg=(AGMessage*)_allocsafe(sizeof(AGMessage));
            agmsg->type=AG_STRINGGADGET_OK;
            agmsg->stringgadget.sg=this;
            aguix->putAGMsg(agmsg);
          } else {
            AGMessage *agmsg=(AGMessage*)_allocsafe(sizeof(AGMessage));
            agmsg->type=AG_STRINGGADGET_CANCEL;
            agmsg->stringgadget.sg=this;
            aguix->putAGMsg(agmsg);
          }
        }
      } else if(msg->type==ButtonRelease) {
        applySelection();
      } else if(E->type==MotionNotify) {
        aguix->queryPointer(msg->window,&tx,&ty);
        int nc=tx/cw;
        int ncp=nc+xoffset;
        if(ncp>=(int)strlen(text)) ncp=strlen(text);
        if(ncp<0) ncp=0;
	cursorpos=ncp;
	if((nc<0)&&(xoffset>0)) {
	  xoffset--;
	  cursorpos=xoffset;
	} else if((nc>=maxdisplay)&&(((int)strlen(text)-xoffset)>=maxdisplay)) {
	  xoffset++;
	  cursorpos=xoffset+maxdisplay-1;
	}
	selend=cursorpos;
        redraw();
      }
    }
  } else if(msg->type==ButtonPress) {
    if(msg->window==twin) {
      active=true;
//      returnvalue=true;
      AGMessage *agmsg=(AGMessage*)_allocsafe(sizeof(AGMessage));
      agmsg->type=AG_STRINGGADGET_ACTIVATE;
      agmsg->stringgadget.sg=this;
      aguix->putAGMsg(agmsg);
      if(forbidPosChange==false) {
        aguix->queryPointer(msg->window,&tx,&ty);
        int nc=tx/cw;
        cursorpos=nc+xoffset;
        if(cursorpos>=(int)strlen(text)) cursorpos=(int)strlen(text);
        if(cursorpos<0) cursorpos=0;
        selstart=selend=cursorpos;
        if(msg->button==Button2) insertSelection();
      }
      redraw();
    }
  }
  if(msg->type==Expose) {
    if(msg->window==win) bredraw();
    else if(msg->window==twin) tredraw();
  }
  return returnvalue;
}

void StringGadget::setText(const char *new_text)
{
  _freesafe(text);
  text=dupstring(new_text);
  if((xoffset+maxdisplay)>=(int)strlen(new_text)) {
    xoffset=(int)strlen(new_text)-maxdisplay;
    if(xoffset<0) xoffset=0;
  }
  if(cursorpos>=(int)strlen(new_text)) {
    cursorpos=(int)strlen(new_text);
  }
  selstart=selend=cursorpos;
  tredraw();
}

char *StringGadget::getText()
{
  return text;
}

bool StringGadget::isActive()
{
  return active;
}

void StringGadget::updateWin()
{
  if(parent!=NULL) {
    parent->resizeSubWin(twin,w-6,h-6);
    if(font==NULL) maxdisplay=(w-6-2)/aguix->getCharWidth();
    else maxdisplay=(w-6-2)/font->getCharWidth();
    tredraw();
  }
}

void StringGadget::setParent(AWindow *parent)
{
  this->parent=parent;
  win=parent->getSubWindow(0,x,y,w,h);
  twin=parent->getSubWindow(win,3,3,w-6,h-6);
  if(font==NULL) maxdisplay=(w-6-2)/aguix->getCharWidth();
  else maxdisplay=(w-6-2)/font->getCharWidth();
}

void StringGadget::activate()
{
  active=true;
  ignoreRelease=true;
  tredraw();
}

void StringGadget::deactivate()
{
  active=false;
  tredraw();
}

void StringGadget::applySelection()
{
  int x1,x2;
  x1=(selstart<=selend?selstart:selend);
  x2=(selstart>selend?selstart:selend);
  if(x2==(int)strlen(text)) x2--;
  if(x1<x2) {
    XStoreBytes(aguix->getDisplay(),text+x1,x2-x1+1);
    aguix->Flush();
  }
}

void StringGadget::removeSelection()
{
  int x1,x2;
  x1=(selstart<=selend?selstart:selend);
  x2=(selstart>selend?selstart:selend);
  if(x2==(int)strlen(text)) x2--;
  if(x1<x2) {
    char *tstr;
    tstr=(char*)_allocsafe(strlen(text)-x2+x1+1);
    strncpy(tstr,text,x1);
    tstr[x1]=0;
    strcat(tstr,text+x2+1);
    _freesafe(text);
    text=tstr;
    selstart=selend=cursorpos=x1;
    if(cursorpos<xoffset) xoffset=cursorpos;
  }
}

void StringGadget::insertSelection()
{
  Display *display=aguix->getDisplay();
  int nbytes_return;
  char *bytes_return,*tstr;
  bytes_return=XFetchBytes(display,&nbytes_return);
  if((nbytes_return>0)&&(bytes_return)) {
    // an Cursorpos einfgen
    int tl=strlen(text);
    tstr=(char*)_allocsafe(tl+nbytes_return+2);
    strncpy(tstr,text,cursorpos);
    tstr[cursorpos]=0;
    strncat(tstr,bytes_return,nbytes_return);
    tstr[cursorpos+nbytes_return]=0;
    strcat(tstr,text+cursorpos);
    _freesafe(text);
    text=tstr;
  }
  if(bytes_return) XFree(bytes_return);
}

int StringGadget::setFont(char *fontname)
{
  font=aguix->getFont(fontname);
  if(font==NULL) return -1;
  return 0;
}

const char *StringGadget::getType()
{
  return type;
}

bool StringGadget::isType(const char *qtype)
{
  if(strcmp(type,qtype)==0) return true;
  return false;
}

bool StringGadget::isParent(Window child)
{
  if(child==win) return true;
  if(child==twin) return true;
  return false;
}

int StringGadget::getXOffset()
{
  return xoffset;
}

int StringGadget::getCursor()
{
  return cursorpos;
}

bool StringGadget::isPosChangeForbidden()
{
  return forbidPosChange;
}

void StringGadget::setForbidPosChange(bool nv)
{
  forbidPosChange=nv;
}

void StringGadget::setStrongKeyCapture(bool nv)
{
  strongkeycapture=nv;
}
