#include "text.h"
#include "lowlevelfunc.h"
#include "awindow.h"

const char *Text::type="Text";

Text::Text(AGUIX *aguix):GUIElement(aguix)
{
  text=(char*)_allocsafe(1);
  text[0]=0;
  color=1;
  bgset=false;
  font=NULL;
}

Text::Text(AGUIX *aguix,int x,int y,const char *text,int color):GUIElement(aguix)
{
  this->text=dupstring(text);
  this->x=x;
  this->y=y;
  this->w=aguix->getCharWidth()*strlen(text)+1;
  this->h=aguix->getCharHeight();
  this->color=color;
  bgset=false;
  font=NULL;
}

Text::~Text()
{
  if(text!=NULL) _freesafe(text);
  if(parent!=NULL) {
    parent->removeSubWin(win);
  }
}

char *Text::getText()
{
  return dupstring(text);
}

void Text::setText(const char *new_text)
{
  if(text!=NULL) _freesafe(text);
  text=dupstring(new_text);
  if(font==NULL) this->w=aguix->getCharWidth()*strlen(text)+1;
  else this->w=font->getCharWidth()*strlen(text)+1;
  if(parent!=NULL) {
    parent->resizeSubWin(win,w,h);
  }
  redraw();
}

int Text::getColor()
{
  return color;
}

void Text::setColor(int color)
{
  if(color<(aguix->getMaxCols())) {
    this->color=color;
    redraw();
  }
}

void Text::redraw()
{
  GC usegc;
  if(font==NULL) usegc=0; else usegc=font->getGC();
  if(bgset==false) {
    aguix->SetWindowBG(win,parent->getBG());
    bgset=true;
  }
  aguix->ClearWin(win);
  aguix->setFG(usegc,color);
  if(font==NULL) aguix->DrawText(win,text,0,0);
  else aguix->DrawText(win,font,text,0,0);
}

void Text::flush()
{
}

bool Text::handleMessage(XEvent *E,Message *msg)
{
  if((msg->type==Expose)&&(msg->window==win)) {
    redraw();
  }
  return false;
}

int Text::setFont(char *fontname)
{
  font=aguix->getFont(fontname);
  if(font==NULL) return -1;
  this->w=font->getCharWidth()*strlen(text)+1;
  this->h=font->getCharHeight();
  if(parent!=NULL) {
    parent->resizeSubWin(win,w,h);
  }
  return 0;
}

const char *Text::getType()
{
  return type;
}

bool Text::isType(const char *qtype)
{
  if(strcmp(type,qtype)==0) return true;
  return false;
}
